package main

import (
	"database/sql"
	"net"

	migration "bitbucket.org/prmunty/util/dbmigration/db"
	"bitbucket.org/prmunty/util/ethernet"

	_ "github.com/mattn/go-sqlite3"
)

func main() {
	const version float64 = 4.0

	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		return migration.DoUntilErr(tx,
			UpdateEquipmentTable,
			AddFreezeScreenSelections,
			CreateModeOrdinalsTable,
			InsertModeOrdinals,
			UpdateFreezeTable,
			InsertFreezeData,
		)
	})

}

func UpdateEquipmentTable(tx *sql.Tx) error {
	rows, err := tx.Query(
		`SELECT ip FROM equipment`,
	);

	if err != nil {
		return err
	}

	for rows.Next() {
		var ip string

		err = rows.Scan(&ip)
		if err != nil {
			return err
		}

		netIp := net.ParseIP(ip)

		if ethernet.IsLocalIP(netIp) {
			if _, err = tx.Exec(
				`DELETE FROM equipment WHERE ip = ?`,
				ip,
			); err != nil {
				return err
			}
		}
	}

	return nil
}

func AddFreezeScreenSelections(tx *sql.Tx) error {
	freezeIDs := []int{4439,4440}

	for _, freezeID := range freezeIDs {
		if _, err := tx.Exec(
			`INSERT OR IGNORE INTO freeze_data_compressor(freeze_id, selected) VALUES( ?, 0 )`,
			freezeID,
		); err != nil {
			return err
		}
	}

	return nil
}

func CreateModeOrdinalsTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS mode_ordinals; CREATE TABLE mode_ordinals(mode_data Integer,text_index Integer,ordinal Integer)`,
	); err != nil {
		return err
	}

	return nil
}

func InsertModeOrdinals(tx *sql.Tx) error {
	userdefinedIds := map[int]int{
		0:1,
		1:2,
		2:3,
		3:4,
		4:5,
		5:6,
		6:7,
		7:8,
		8:9,
		9:10,
		10:11,
		11:12,
		12:13,
		13:14,
		14:15,
		16:16,
		17:17,
		20:36,
		21:37,
		25:18,
		26:19,
		27:20,
		28:21,
		29:22,
		30:23,
		31:42,
		32:43,
		33:44,
		34:45,
		35:46,
		36:47,
		37:48,
		38:49,
		39:50,
		40:51,
		41:52,
		42:53,
		43:54,
		44:55,
		45:56,
		46:57,
		47:58,
		48:59,
		49:60,
		50:61,
		67:24,
		68:25,
		70:26,
		71:27,
		72:28,
		73:29,
		74:30,
		77:31,
		78:32,
		79:33,
		80:34,
		81:35,
		98:38,
		101:39,
		102:40,
		103:41,
	}

	trendingIds := map[int]int{
		0:1,
		1:2,
		2:3,
		3:4,
		4:5,
		5:6,
		6:7,
		7:8,
		8:9,
		9:10,
		10:11,
		11:12,
		12:13,
		13:14,
		14:15,
		16:16,
		17:17,
		20:36,
		21:37,
		25:18,
		26:19,
		27:20,
		28:21,
		29:22,
		30:23,
		31:42,
		32:43,
		33:44,
		34:45,
		35:46,
		36:47,
		37:48,
		38:49,
		39:50,
		40:51,
		41:52,
		42:53,
		43:54,
		44:55,
		45:56,
		46:57,
		47:58,
		48:59,
		49:60,
		50:61,
		67:24,
		68:25,
		70:26,
		71:27,
		72:28,
		73:29,
		74:30,
		77:31,
		78:32,
		79:33,
		80:34,
		81:35,
		98:38,
		101:39,
		102:40,
		103:41,
	}

	for key, value := range userdefinedIds {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			311,
			key,
			value,
		); err != nil {
			return err
		}
	}

	for key, value := range trendingIds {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			369,
			key,
			value,
		); err != nil {
			return err
		}
	}

	//For control inputs
	for key, value := range userdefinedIds {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			572,
			key,
			value,
		); err != nil {
			return err
		}
	}

	return nil
}

func UpdateFreezeTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS "freeze_data_compressor"; CREATE TABLE "freeze_data_compressor" ("freeze_id" Integer PRIMARY KEY, "table_id" Integer, "selected" Boolean,"ordinal" Integer);`,
	); err != nil {
		return err
	}

	return nil
}


func InsertFreezeData(tx *sql.Tx) error {
	freezeIds := map[int]int {
    12:102,
    13:5,
    14:14,
    15:96,
    16:15,
    21:98,
    27:20,
    29:22,
    132:21,
    134:42,
    135:43,
    136:44,
    137:45,
    138:46,
    139:47,
    140:48,
    141:49,
    142:50,
    143:51,
    144:52,
    145:53,
    146:54,
    147:55,
    148:56,
    149:57,
    150:58,
    151:59,
    152:60,
    153:61,
    507:97,
    618:8,
    620:9,
    739:11,
    761:69,
    762:70,
    763:71,
    764:72,
    765:73,
    766:74,
    767:75,
    768:76,
    771:3,
    772:4,
    774:6,
    775:7,
    778:10,
    780:12,
    781:13,
    1082:27,
    1083:28,
    1084:29,
    1085:30,
    1179:24,
    1512:63,
    1513:64,
    1531:25,
    1565:31,
    1566:32,
    1567:33,
    1568:34,
    1569:35,
    1596:19,
    1666:26,
    1743:77,
    1744:78,
    1745:79,
    2153:67,
    2162:38,
    2174:62,
    2195:66,
    2212:65,
    2213:39,
    2214:40,
    2215:68,
    2219:41,
    2227:1,
    2228:2,
    2320:16,
    2321:17,
    2542:23,
    2923:18,
    4203:80,
    4204:81,
    4205:82,
    4206:83,
    4207:84,
    4208:85,
    4209:86,
    4210:87,
    4211:88,
    4212:89,
    4213:90,
    4214:91,
    4215:92,
    4216:93,
    4217:94,
    4218:95,
    4244:99,
    4245:100,
    4246:101,
    4439:36,
    4440:37,
  }

	modeIds := []int {
		21, 4244, 4245, 4246,
	}

	calcIds := []int {
		12, 15,
	}

	timerIds := []int {
		507,
	}

	for key, value := range freezeIds {
		table_id := 0
		found := false

		if !found {
			for index, _ := range modeIds {
				if modeIds[index] == key {
					table_id = 3
					found = true
				}
			}
		}

		if !found {
			for index, _ := range calcIds {
				if modeIds[index] == key {
					table_id = 2
					found = true
				}
			}
		}

		if !found {
			for index, _ := range timerIds {
				if modeIds[index] == key {
					table_id = 6
					found = true
				}
			}
		}

		if _, err := tx.Exec(
			"INSERT INTO \"freeze_data_compressor\" (\"freeze_id\",\"table_id\",\"selected\",\"ordinal\") VALUES(?, ?, ?, ?);",
			key,
			table_id,
			1,
			value,
		); err != nil {
			return err
		}
	}

	return nil
}
